"""Abstract for base API class."""
import os
from typing import Union, Any

from csp_cbc_auth.cbc.session import CBCSession
from psc_auth_integration.entities.auth_user_session import AuthUserSession

from cds_configuration_service.logger import configure_logging

LOG = configure_logging(__name__)


class ConfigurationServiceBaseClass:
    """
    Base class for making API requests.
    Handles common operations, including request sending, logging, and error handling.
    """

    def __init__(self, session: Union[AuthUserSession, CBCSession]):
        self.session = session
        self.resources = os.path.join(os.path.dirname(os.path.realpath(__file__)), '../', 'resources')
        self.configuration_service_base_url = f'{session.gateway_url}/cdsconfigurationservice'

    def get(self, url: str, **kwargs):
        response = self.session.get(url=url, **kwargs)
        LOG.info('GET: %s Status: %s', url, response.status_code)
        return response

    def post(self, url: str, data: Any = None, json: dict = None, **kwargs):
        response = self.session.post(url=url, data=data, json=json, **kwargs)
        LOG.info('POST: %s Status: %s', url, response.status_code)
        return response

    def patch(self, url: str, data: Any = None, **kwargs):
        response = self.session.patch(url=url, data=data, **kwargs)
        LOG.info('PATCH: %s Status: %s', url, response.status_code)
        return response

    def delete(self, url: str, **kwargs):
        response = self.session.delete(url=url, **kwargs)
        LOG.info('DELETE: %s Status: %s', url, response.status_code)
        return response
