import pytest

from cds_configuration_service.tests.utils.auth import CDSSession
from cds_configuration_service.tests.plugins.ownership_plugin import SGUID
from cds_configuration_service.api_clients.custom_configuration_service_v1 import CustomConfigurationServiceV1Api


@pytest.mark.sguid(SGUID.psc_content_mgmt_app_cds_configuration_service)
class TestCustomProfileManagement:
    PROFILE_KEY_SINGLE_DATE = 'TEST_ORG:TEST_GROUP_ID:20230714:TEST'
    PROFILE_KEY_DATE_RANGE = 'TEST_ORG:TEST_GROUP_ID:20230714_20230814:TEST'
    ORG_KEY = 'TEST_ORG'
    GROUP_KEY = 'TEST_GROUP_ID'

    @staticmethod
    def create_new_date_range_profile_data():
        return {
            'group_id': 'TEST_GROUP_ID',
            'profile_date_range': {
                'date_from': '20230714',
                'date_to': '20230814'
            },
            'content_identifiers': [
                'TEST'
            ],
            'budget': [
                [50.0, 50.0, 1.0, 1.0, 1.0, 1.0, 1.0, -1.0, 1.0, 0.0, 0.0, 0.0, 1.0, 100.0, 100.0, 10.0, 1.0, 100.0,
                 100.0, 1.0, 1.0, 10.0, 1.0, 1.0],
                [50.0, 50.0, 1.0, 1.0, 1.0, 1.0, 1.0, -1.0, 1.0, 0.0, 0.0, 0.0, 1.0, 100.0, 100.0, 100.0, 1.0, 100.0,
                 100.0, 1.0, 1.0, 10.0, 1.0, 1.0],
                [50.0, 50.0, 1.0, 1.0, 1.0, 1.0, 1.0, -1.0, 1.0, 0.0, 0.0, 0.0, 1.0, 100.0, 100.0, 100.0, 1.0, 100.0,
                 100.0, 1.0, 1.0, 10.0, 1.0, 1.0],
                [50.0, 50.0, 1.0, 1.0, 1.0, 1.0, 1.0, -1.0, 1.0, 0.0, 0.0, 0.0, 1.0, 100.0, 100.0, 10.0, 1.0, 10.0,
                 100.0, 1.0, 1.0, 10.0, 1.0, 1.0],
                [50.0, 50.0, 1.0, 1.0, 1.0, 1.0, 1.0, -1.0, 1.0, 0.0, 0.0, 0.0, 1.0, 100.0, 100.0, 100.0, 1.0, 100.0,
                 100.0, 1.0, 1.0, 10.0, 1.0, 1.0],
                [50.0, 50.0, 1.0, 1.0, 1.0, 1.0, 1.0, -1.0, 1.0, 0.0, 0.0, 0.0, 1.0, 100.0, 100.0, 100.0, 1.0, 100.0,
                 100.0, 1.0, 1.0, 10.0, 1.0, 1.0],
                [50.0, 50.0, 1.0, 1.0, 1.0, 1.0, 1.0, -1.0, 1.0, 0.0, 0.0, 0.0, 1.0, 100.0, 100.0, 100.0, 1.0, 100.0,
                 100.0, 1.0, 1.0, 10.0, 1.0, 1.0]]
        }

    @staticmethod
    def create_new_single_date_profile_data():
        return {
            'group_id': 'TEST_GROUP_ID',
            'profile_date': '20230714',
            'content_identifiers': [
                'TEST'
            ],
            'budget': [
                [50.0, 50.0, 1.0, 1.0, 1.0, 1.0, 1.0, -1.0, 1.0, 0.0, 0.0, 0.0, 1.0, 100.0, 100.0, 10.0, 1.0, 100.0,
                 100.0, 1.0, 1.0, 10.0, 1.0, 1.0],
                [50.0, 50.0, 1.0, 1.0, 1.0, 1.0, 1.0, -1.0, 1.0, 0.0, 0.0, 0.0, 1.0, 100.0, 100.0, 100.0, 1.0, 100.0,
                 100.0, 1.0, 1.0, 10.0, 1.0, 1.0],
                [50.0, 50.0, 1.0, 1.0, 1.0, 1.0, 1.0, -1.0, 1.0, 0.0, 0.0, 0.0, 1.0, 100.0, 100.0, 100.0, 1.0, 100.0,
                 100.0, 1.0, 1.0, 10.0, 1.0, 1.0],
                [50.0, 50.0, 1.0, 1.0, 1.0, 1.0, 1.0, -1.0, 1.0, 0.0, 0.0, 0.0, 1.0, 100.0, 100.0, 10.0, 1.0, 10.0,
                 100.0, 1.0, 1.0, 10.0, 1.0, 1.0],
                [50.0, 50.0, 1.0, 1.0, 1.0, 1.0, 1.0, -1.0, 1.0, 0.0, 0.0, 0.0, 1.0, 100.0, 100.0, 100.0, 1.0, 100.0,
                 100.0, 1.0, 1.0, 10.0, 1.0, 1.0],
                [50.0, 50.0, 1.0, 1.0, 1.0, 1.0, 1.0, -1.0, 1.0, 0.0, 0.0, 0.0, 1.0, 100.0, 100.0, 100.0, 1.0, 100.0,
                 100.0, 1.0, 1.0, 10.0, 1.0, 1.0],
                [50.0, 50.0, 1.0, 1.0, 1.0, 1.0, 1.0, -1.0, 1.0, 0.0, 0.0, 0.0, 1.0, 100.0, 100.0, 100.0, 1.0, 100.0,
                 100.0, 1.0, 1.0, 10.0, 1.0, 1.0]]
        }

    def test_custom_profile_workflow(self, auth_session: CDSSession,
                                     custom_configuration_service_v1_api: CustomConfigurationServiceV1Api):
        """
        Test the workflow of creating, updating, and deleting a custom profile.

        This test performs a complete workflow involved in the custom configuration service.
        The steps are:
        1. Verifying that the target profile does not initially exist.
        2. Creating new profiles with different configurations (single date and date range).
        3. Verifying that the new profiles are created successfully.
        4. Updating one of the newly created profiles.
        5. Verifying that the profile update is successful.
        6. Deleting the newly created profiles.
        7. Verifying that the profiles are deleted successfully.

        The test ensures that all operations (CRUD) on custom profiles are working as expected
        in the CustomConfigurationServiceV1Api. It includes checks for successful responses
        (status code 200) from the service and specific data checks (like comparing budget values).

        Any created profile is cleaned up at the end of the test to ensure that the test environment
        remains consistent for future tests, regardless of the test result.

        Parameters:
        auth_session (CDSSession): A session object representing an authenticated user.
        custom_configuration_service_v1_api (CustomConfigurationServiceV1Api): An API wrapper to be tested.

        The test will fail if any of the responses from the service do not match the expected values
        or if the profile data does not reflect the intended updates.
        """
        try:
            # STEP 1: Verify target profile does not exist
            response = custom_configuration_service_v1_api.get_all_profiles('TEST_ORG')
            assert response.status_code == 404

            # STEP 2: Create new profiles with single date and date range
            profile_data_single_date = self.create_new_single_date_profile_data()
            create_response = custom_configuration_service_v1_api.add_profile(self.ORG_KEY, profile_data_single_date)
            create_response.raise_for_status()

            profile_data_date_range = self.create_new_date_range_profile_data()
            create_response = custom_configuration_service_v1_api.add_profile(self.ORG_KEY, profile_data_date_range)
            create_response.raise_for_status()

            # STEP 3: Verify creation
            new_profile = self.retrieve_profiles_by_keys(custom_configuration_service_v1_api, self.ORG_KEY,
                                                         [self.PROFILE_KEY_SINGLE_DATE, self.PROFILE_KEY_DATE_RANGE])
            assert len(new_profile) == 2

            # STEP 4: Update profile
            update_data = {
                'budget': profile_data_single_date.get('budget')
            }
            update_data['budget'][0][0] = 100.0
            update_response = custom_configuration_service_v1_api.update_profile(self.ORG_KEY,
                                                                                 self.PROFILE_KEY_SINGLE_DATE,
                                                                                 update_data.get('budget'))
            update_response.raise_for_status()

            # STEP 5: Verify update
            updated_profile = self.retrieve_profile_by_key(custom_configuration_service_v1_api, self.ORG_KEY,
                                                           self.PROFILE_KEY_SINGLE_DATE)
            assert updated_profile['budget'][0][0] == 100.0

            # STEP 6: Delete profile
            delete_response_single_date = custom_configuration_service_v1_api.delete_profile(
                self.ORG_KEY, self.PROFILE_KEY_SINGLE_DATE)
            delete_response_single_date.raise_for_status()
            delete_response_date_range = custom_configuration_service_v1_api.delete_profile(
                self.ORG_KEY, self.PROFILE_KEY_DATE_RANGE)
            delete_response_date_range.raise_for_status()

            # STEP 7: Verify deletion
            response = custom_configuration_service_v1_api.get_all_profiles('TEST_ORG')
            assert response.status_code == 404

        finally:
            # Cleanup: Irrespective of test result, ensure the profile is deleted for cleanup
            delete_response_single_date = custom_configuration_service_v1_api.delete_profile(
                self.ORG_KEY, self.PROFILE_KEY_SINGLE_DATE)
            delete_response_date_range = custom_configuration_service_v1_api.delete_profile(
                self.ORG_KEY, self.PROFILE_KEY_DATE_RANGE)
            assert delete_response_single_date.status_code in [200, 404]
            assert delete_response_date_range.status_code in [200, 404]

    def retrieve_profiles_by_keys(self, api, org_key, profile_keys):
        """
        Helper method to retrieve profiles by a list of keys from the complete list of profiles.
        Checks if all specified profile keys exist in the retrieved profiles.
        """
        response = api.get_all_profiles(org_key)
        response.raise_for_status()

        # Finding profiles that match the provided keys
        found_profiles = [profile for profile in response.json() if profile.get('profile_key') in profile_keys]

        # Verify if the number of found profiles matches the number of keys provided
        all_exist = len(found_profiles) == len(profile_keys)

        return all_exist, found_profiles

    def retrieve_profile_by_key(self, api, org_key, profile_key):
        """Helper method to retrieve a profile by key from the complete list of profiles."""
        response = api.get_profile(org_key, profile_key)
        response.raise_for_status()
        return response.json()
